@@ -143,8 +143,10 @@ class MotionEvent(object):
          'px', 'py', 'pz',
          # delta from the last position and current one, in screen range
          'dx', 'dy', 'dz',
-         'time_start', 'is_double_tap',
-         'double_tap_time', 'ud')
+         'time_start',
+         'is_double_tap', 'double_tap_time',
+         'is_triple_tap', 'triple_tap_time',
+         'ud')
 
     def __init__(self, device, id, args):
         if self.__class__ == MotionEvent:
@@ -173,7 +175,7 @@ class MotionEvent(object):
         self.grab_exclusive_class = None
         self.grab_state = False
 
-        #: Used to determine which widget the touch is beeing dispatched.
+        #: Used to determine which widget the touch is being dispatched.
         #: Check :func:`grab` function for more information.
         self.grab_current = None
 
@@ -253,10 +255,19 @@ class MotionEvent(object):
         #: Indicate if the touch is a double tap or not
         self.is_double_tap = False
 
+        #: Indicate if the touch is a triple tap or not
+        #: .. versionadded:: 1.7.0
+        self.is_triple_tap = False
+
         #: If the touch is a :attr:`is_double_tap`, this is the time between the
         #: previous tap and the current touch.
         self.double_tap_time = 0
 
+        #: If the touch is a :attr:`is_triple_tap`, this is the time between the
+        #: first tap and the current touch.
+        #: .. versionadded:: 1.7.0
+        self.triple_tap_time = 0
+
         #: User data dictionnary. Use this dictionnary to save your own data on
         #: the touch.
         self.ud = EnhancedDictionnary()
